@@ -6,8 +6,13 @@
 #include <rz_io.h>
 #include <rz_cons.h>
 #include <rz_list.h>
+#include <ht_pu.h>
+#include <rz_demangler.h>
 
 typedef struct rz_bin_t RzBin;
+typedef struct rz_bin_file_t RzBinFile;
+typedef struct rz_bin_source_line_info_t RzBinSourceLineInfo;
+typedef struct rz_bin_reloc_storage_t RzBinRelocStorage;
 
 #include <rz_bin_dwarf.h>
 #include <rz_pdb.h>
@@ -70,6 +75,8 @@ RZ_LIB_VERSION_HEADER(rz_bin);
 #define RZ_BIN_REQ_SIGNATURE        0x80000000
 #define RZ_BIN_REQ_TRYCATCH         0x100000000
 #define RZ_BIN_REQ_SECTIONS_MAPPING 0x200000000
+#define RZ_BIN_REQ_CLASSES_SOURCES  0x400000000
+#define RZ_BIN_REQ_BASEFIND         0x800000000
 
 /* RzBinSymbol->method_flags : */
 #define RZ_BIN_METH_CLASS                 0x0000000000000001L
@@ -103,11 +110,14 @@ RZ_LIB_VERSION_HEADER(rz_bin);
 #define RZ_BIN_BIND_HIOS_STR    "HIOS"
 #define RZ_BIN_BIND_LOPROC_STR  "LOPROC"
 #define RZ_BIN_BIND_HIPROC_STR  "HIPROC"
+#define RZ_BIN_BIND_IMPORT_STR  "IMPORT"
 #define RZ_BIN_BIND_UNKNOWN_STR "UNKNOWN"
 
 #define RZ_BIN_TYPE_NOTYPE_STR      "NOTYPE"
 #define RZ_BIN_TYPE_OBJECT_STR      "OBJ"
 #define RZ_BIN_TYPE_FUNC_STR        "FUNC"
+#define RZ_BIN_TYPE_FIELD_STR       "FIELD"
+#define RZ_BIN_TYPE_IFACE_STR       "IFACE"
 #define RZ_BIN_TYPE_METH_STR        "METH"
 #define RZ_BIN_TYPE_STATIC_STR      "STATIC"
 #define RZ_BIN_TYPE_SECTION_STR     "SECT"
@@ -122,39 +132,50 @@ RZ_LIB_VERSION_HEADER(rz_bin);
 #define RZ_BIN_TYPE_SPECIAL_SYM_STR "SPCL"
 #define RZ_BIN_TYPE_UNKNOWN_STR     "UNK"
 
-enum {
-	RZ_BIN_SYM_ENTRY,
-	RZ_BIN_SYM_INIT,
-	RZ_BIN_SYM_MAIN,
-	RZ_BIN_SYM_FINI,
-	RZ_BIN_SYM_LAST
-};
+typedef enum {
+	RZ_BIN_SPECIAL_SYMBOL_ENTRY,
+	RZ_BIN_SPECIAL_SYMBOL_INIT,
+	RZ_BIN_SPECIAL_SYMBOL_MAIN,
+	RZ_BIN_SPECIAL_SYMBOL_FINI,
+	RZ_BIN_SPECIAL_SYMBOL_LAST
+} RzBinSpecialSymbol;
 
 // name mangling types
-// TODO: Rename to RZ_BIN_LANG_
-enum {
-	RZ_BIN_NM_NONE = 0,
-	RZ_BIN_NM_JAVA = 1,
-	RZ_BIN_NM_C = 1 << 1,
-	RZ_BIN_NM_GO = 1 << 2,
-	RZ_BIN_NM_CXX = 1 << 3,
-	RZ_BIN_NM_OBJC = 1 << 4,
-	RZ_BIN_NM_SWIFT = 1 << 5,
-	RZ_BIN_NM_DLANG = 1 << 6,
-	RZ_BIN_NM_MSVC = 1 << 7,
-	RZ_BIN_NM_RUST = 1 << 8,
-	RZ_BIN_NM_KOTLIN = 1 << 9,
-	RZ_BIN_NM_BLOCKS = 1 << 31,
-	RZ_BIN_NM_ANY = -1,
-};
+typedef enum {
+	RZ_BIN_LANGUAGE_UNKNOWN = 0,
+	RZ_BIN_LANGUAGE_JAVA = 1,
+	RZ_BIN_LANGUAGE_C = 1 << 1,
+	RZ_BIN_LANGUAGE_GO = 1 << 2,
+	RZ_BIN_LANGUAGE_CXX = 1 << 3,
+	RZ_BIN_LANGUAGE_OBJC = 1 << 4,
+	RZ_BIN_LANGUAGE_SWIFT = 1 << 5,
+	RZ_BIN_LANGUAGE_DLANG = 1 << 6,
+	RZ_BIN_LANGUAGE_MSVC = 1 << 7,
+	RZ_BIN_LANGUAGE_RUST = 1 << 8,
+	RZ_BIN_LANGUAGE_KOTLIN = 1 << 9,
+	RZ_BIN_LANGUAGE_GROOVY = 1 << 10,
+	RZ_BIN_LANGUAGE_DART = 1 << 11,
+	RZ_BIN_LANGUAGE_BLOCKS = 1 << 31,
+} RzBinLanguage;
+
+#define RZ_BIN_LANGUAGE_MASK(x)       ((x) & ~RZ_BIN_LANGUAGE_BLOCKS)
+#define RZ_BIN_LANGUAGE_HAS_BLOCKS(x) ((x)&RZ_BIN_LANGUAGE_BLOCKS)
 
 enum {
-	RZ_STRING_TYPE_DETECT = '?',
-	RZ_STRING_TYPE_ASCII = 'a',
-	RZ_STRING_TYPE_UTF8 = 'u',
-	RZ_STRING_TYPE_WIDE = 'w', // utf16 / widechar string
-	RZ_STRING_TYPE_WIDE32 = 'W', // utf32
-	RZ_STRING_TYPE_BASE64 = 'b',
+	RZ_BIN_STRING_ENC_DETECT = 'g',
+	RZ_BIN_STRING_ENC_8BIT = 'b', // unknown 8bit encoding but with ASCII from 0 to 0x7f
+	RZ_BIN_STRING_ENC_UTF8 = '8',
+	RZ_BIN_STRING_ENC_MUTF8 = 'm', // modified utf8
+	RZ_BIN_STRING_ENC_WIDE_LE = 'u', // utf16 / widechar string
+	RZ_BIN_STRING_ENC_WIDE32_LE = 'U', // utf32
+	RZ_BIN_STRING_ENC_WIDE_BE = 'n', // utf16-be / widechar string
+	RZ_BIN_STRING_ENC_WIDE32_BE = 'N', // utf32-be
+	RZ_BIN_STRING_ENC_BASE64 = '6',
+	RZ_BIN_STRING_ENC_IBM037 = 'c',
+	RZ_BIN_STRING_ENC_IBM290 = 'd',
+	RZ_BIN_STRING_ENC_EBCDIC_UK = 'k',
+	RZ_BIN_STRING_ENC_EBCDIC_US = 's',
+	RZ_BIN_STRING_ENC_EBCDIC_ES = 't',
 };
 
 enum {
@@ -164,12 +185,13 @@ enum {
 	RZ_BIN_CLASS_PROTECTED,
 };
 
-enum {
+typedef enum {
 	RZ_BIN_RELOC_8 = 8,
 	RZ_BIN_RELOC_16 = 16,
+	RZ_BIN_RELOC_24 = 24,
 	RZ_BIN_RELOC_32 = 32,
 	RZ_BIN_RELOC_64 = 64
-};
+} RzBinRelocType;
 
 enum {
 	RZ_BIN_TYPE_DEFAULT = 0,
@@ -227,7 +249,6 @@ typedef struct rz_bin_info_t {
 	int has_crypto;
 	int has_nx;
 	int big_endian;
-	bool has_lit;
 	char *actual_checksum;
 	char *claimed_checksum;
 	int pe_overlay;
@@ -239,46 +260,60 @@ typedef struct rz_bin_info_t {
 	char *compiler;
 } RzBinInfo;
 
+typedef struct rz_bin_file_load_options_t {
+	ut64 baseaddr; ///< where the linker maps the binary in memory
+	ut64 loadaddr; ///< starting physical address to read from the target file
+	bool patch_relocs; ///< ask the bin plugin to fill relocs with valid contents for analysis
+	bool big_endian; ///< only used for binary formats that do not specify the endian in the file, but need it to load, otherwise ignored.
+	bool elf_load_sections; ///< ELF specific, load or not ELF sections
+	bool elf_checks_sections; ///< ELF specific, checks or not ELF sections
+	bool elf_checks_segments; ///< ELF specific, checks or not ELF sections
+} RzBinObjectLoadOptions;
+
 typedef struct rz_bin_object_t {
-	ut64 baddr;
+	RzBinObjectLoadOptions opts;
 	st64 baddr_shift;
-	ut64 loadaddr;
 	ut64 boffset;
 	ut64 size;
 	ut64 obj_size;
+	RzList /*<RzBinVirtualFile>*/ *vfiles;
+	RzList /*<RzBinMap>*/ *maps;
 	RzList /*<RzBinSection>*/ *sections;
 	RzList /*<RzBinImport>*/ *imports;
 	RzList /*<RzBinSymbol>*/ *symbols;
-	RzList /*<??>*/ *entries;
-	RzList /*<??>*/ *fields;
-	RzList /*<??>*/ *libs;
-	RBNode /*<RzBinReloc>*/ *relocs;
-	RzList /*<??>*/ *strings;
+	RzList /*<RzBinResource>*/ *resources;
+	/**
+	 * \brief Acceleration structure for fast access of the symbol for a given import.
+	 * This associates the name of every symbol where is_imported == true to the symbol itself.
+	 */
+	HtPP /*<const char *, RzBinSymbol>*/ *import_name_symbols; // currently only used for imports, but could be extended to all symbols if needed.
+	RzList /*<RzBinAddr>*/ *entries;
+	RzList /*<RzBinField>*/ *fields;
+	RzList /*<char*>*/ *libs;
+	RzBinRelocStorage *relocs;
+	RzList /*<RzBinString>*/ *strings;
 	RzList /*<RzBinClass>*/ *classes;
 	HtPP *classes_ht;
 	HtPP *methods_ht;
-	RzList /*<RzBinDwarfRow>*/ *lines;
+	RzBinSourceLineInfo *lines;
 	HtUP *strings_db;
-	RzList /*<??>*/ *mem; //RzBinMem maybe?
-	RzList /*<BinMap*/ *maps;
+	RzList /*<RzBinMem>*/ *mem;
 	char *regstate;
 	RzBinInfo *info;
-	RzBinAddr *binsym[RZ_BIN_SYM_LAST];
+	RzBinAddr *binsym[RZ_BIN_SPECIAL_SYMBOL_LAST];
 	struct rz_bin_plugin_t *plugin;
-	int lang;
-	Sdb *kv;
-	Sdb *addrzklassmethod;
+	RzBinLanguage lang;
+	RZ_DEPRECATE RZ_BORROW Sdb *kv; ///< deprecated, put info in C structures instead of this (holds a copy of another pointer.)
+	HtUP *addrzklassmethod;
 	void *bin_obj; // internal pointer used by formats
 } RzBinObject;
 
 // XXX: RbinFile may hold more than one RzBinObject
 /// XX curplugin == o->plugin
-typedef struct rz_bin_file_t {
+struct rz_bin_file_t {
 	char *file;
-	int fd;
+	int fd; ///< when used in combination with RzIO, this refers to the io fd.
 	int size;
-	int rawstr;
-	int strmode;
 	ut32 id;
 	RzBuffer *buf;
 	ut64 offset;
@@ -292,11 +327,9 @@ typedef struct rz_bin_file_t {
 	struct rz_bin_xtr_plugin_t *curxtr;
 	// struct rz_bin_plugin_t *curplugin; // use o->plugin
 	RzList *xtr_data;
-	Sdb *sdb;
-	Sdb *sdb_info;
-	Sdb *sdb_addrinfo;
+	RZ_DEPRECATE Sdb *sdb; ///< deprecated, put info in C structures instead of this
 	struct rz_bin_t *rbin;
-} RzBinFile;
+}; // RzBinFile
 
 typedef struct rz_bin_file_options_t {
 	int rawstr;
@@ -309,16 +342,17 @@ typedef struct rz_bin_file_options_t {
 
 struct rz_bin_t {
 	const char *file;
-	RzBinFile *cur; // TODO: deprecate
+	RZ_DEPRECATE RzBinFile *cur; ///< never use this in new code! Get a file from the binfiles list or track it yourself.
 	int narch;
 	void *user;
+	RzEvent *event;
 	/* preconfigured values */
 	int debase64;
 	int minstrlen;
-	int maxstrlen;
+	int maxstrlen; //< <= 0 means no limit
 	ut64 maxstrbuf;
 	int rawstr;
-	Sdb *sdb;
+	RZ_DEPRECATE Sdb *sdb;
 	RzIDStorage *ids;
 	RzList /*<RzBinPlugin>*/ *plugins;
 	RzList /*<RzBinXtrPlugin>*/ *binxtrs;
@@ -338,12 +372,12 @@ struct rz_bin_t {
 	char *prefix; // bin.prefix
 	char *strenc;
 	ut64 filter_rules;
-	bool demanglercmd;
 	bool verbose;
 	bool use_xtr; // use extract plugins when loading a file?
 	bool use_ldr; // use loader plugins when loading a file?
 	RzStrConstPool constpool;
 	bool is_reloc_patched; // used to indicate whether relocations were patched or not
+	RzDemangler *demangler;
 };
 
 typedef struct rz_bin_xtr_metadata_t {
@@ -361,8 +395,7 @@ typedef struct rz_bin_xtr_extract_t {
 	RzBuffer *buf;
 	ut64 size;
 	ut64 offset;
-	ut64 baddr;
-	ut64 laddr;
+	RzBinObjectLoadOptions obj_opts;
 	int file_count;
 	int loaded;
 	RzBinXtrMetadata *metadata;
@@ -418,26 +451,105 @@ typedef struct rz_bin_trycatch_t {
 RZ_API RzBinTrycatch *rz_bin_trycatch_new(ut64 source, ut64 from, ut64 to, ut64 handler, ut64 filter);
 RZ_API void rz_bin_trycatch_free(RzBinTrycatch *tc);
 
+/**
+ * \brief A single sample of source line info for a specific address
+ *
+ * If at least one of the line, column and file members is not 0/NULL, such a sample specifies the line info
+ * for all addresses greater or equal to address until the next address that has another sample.
+ *
+ * If all the members line, column and file are 0/NULL, then this is a closing sample, indicating that the
+ * previous entry stops here. The address is the first address **not contained** by the previous record.
+ * Such a case corresponds for example to what DW_LNE_end_sequence emits in Dwarf.
+ * Use rz_bin_source_line_sample_is_closing() for checking if a sample is closing.
+ */
+typedef struct rz_bin_source_line_sample_t {
+	/**
+	 * The first address that is covered by the given line and column,
+	 * or, if all other members are 0/NULL, this is the first.
+	 */
+	ut64 address;
+
+	/**
+	 * If > 0, then indicates the line for the given address and the following.
+	 * If == 0, then indicates that no line information is known.
+	 *
+	 * 32bit for this value is an intentional decision to lower memory consumption.
+	 */
+	ut32 line;
+
+	/**
+	 * If > 0, then indicates the column.
+	 * If == 0, then no column information is known.
+	 *
+	 * 32bit for this value is an intentional decision to lower memory consumption.
+	 */
+	ut32 column;
+
+	/**
+	 * Filename, which must come out of the const pool of the owning
+	 * RzBinSourceLineInfo or RzBinSourceLineInfoBuilder.
+	 */
+	const char *file;
+} RzBinSourceLineSample;
+
+/*
+ * see documentation of RzBinSourceLineSample about what closing exactly means.
+ */
+static inline bool rz_bin_source_line_sample_is_closing(const RzBinSourceLineSample *s) {
+	return !s->line && !s->column && !s->file;
+}
+
+struct rz_bin_source_line_info_t {
+	/**
+	 * \brief All source line references for given adresses
+	 *
+	 * These elements must be sorted by address and addresses must be unique, so binary search can be applied.
+	 * Source file information is not contained within this array because source file changes
+	 * are generally much sparser than line changes.
+	 */
+	RzBinSourceLineSample *samples;
+	size_t samples_count;
+	RzStrConstPool filename_pool;
+}; // RzBinSourceLineInfo
+
+/**
+ * Temporary data structure for building an RzBinSourceLineInfo.
+ */
+typedef struct rz_bin_source_line_info_builder_t {
+	RzVector /*<RzBinSourceLineSample>*/ samples; //< may be unsorted and will be sorted in the finalization step
+	RzStrConstPool filename_pool;
+} RzBinSourceLineInfoBuilder;
+
+RZ_API void rz_bin_source_line_info_builder_init(RzBinSourceLineInfoBuilder *builder);
+RZ_API void rz_bin_source_line_info_builder_fini(RzBinSourceLineInfoBuilder *builder);
+RZ_API void rz_bin_source_line_info_builder_push_sample(RzBinSourceLineInfoBuilder *builder, ut64 address, ut32 line, ut32 column, const char *file);
+RZ_API RzBinSourceLineInfo *rz_bin_source_line_info_builder_build_and_fini(RzBinSourceLineInfoBuilder *builder);
+
+RZ_API void rz_bin_source_line_info_free(RzBinSourceLineInfo *sli);
+RZ_API const RzBinSourceLineSample *rz_bin_source_line_info_get_first_at(const RzBinSourceLineInfo *sli, ut64 addr);
+RZ_API const RzBinSourceLineSample *rz_bin_source_line_info_get_next(const RzBinSourceLineInfo *sli, RZ_NONNULL const RzBinSourceLineSample *cur);
+
 typedef struct rz_bin_plugin_t {
 	char *name;
 	char *desc;
 	char *author;
 	char *version;
 	char *license;
-	int (*init)(void *user);
-	int (*fini)(void *user);
-	Sdb *(*get_sdb)(RzBinFile *obj);
-	bool (*load_buffer)(RzBinFile *bf, void **bin_obj, RzBuffer *buf, ut64 loadaddr, Sdb *sdb);
-	ut64 (*size)(RzBinFile *bin); // return ut64 maybe? meh
+	RZ_DEPRECATE Sdb *(*get_sdb)(RzBinFile *obj); ///< deprecated, put info in C structures instead of this
+	bool (*load_buffer)(RzBinFile *bf, RzBinObject *obj, RzBuffer *buf, Sdb *sdb);
+	ut64 (*size)(RzBinFile *bin);
 	void (*destroy)(RzBinFile *bf);
 	bool (*check_bytes)(const ut8 *buf, ut64 length);
 	bool (*check_buffer)(RzBuffer *buf);
+	bool (*check_filename)(const char *filename);
 	ut64 (*baddr)(RzBinFile *bf);
 	ut64 (*boffset)(RzBinFile *bf);
-	RzBinAddr *(*binsym)(RzBinFile *bf, int num);
+	RzList /*<RzBinVirtualFile>*/ *(*virtual_files)(RzBinFile *bf);
+	RzList /*<RzBinMap>*/ *(*maps)(RzBinFile *bf);
+	RzBinAddr *(*binsym)(RzBinFile *bf, RzBinSpecialSymbol num);
 	RzList /*<RzBinAddr>*/ *(*entries)(RzBinFile *bf);
 	RzList /*<RzBinSection>*/ *(*sections)(RzBinFile *bf);
-	RZ_BORROW RzList /*<RzBinDwarfRow>*/ *(*lines)(RzBinFile *bf);
+	RZ_OWN RzBinSourceLineInfo *(*lines)(RzBinFile *bf); //< only called once on load, ownership is transferred to the caller
 	RzList /*<RzBinSymbol>*/ *(*symbols)(RzBinFile *bf);
 	RzList /*<RzBinImport>*/ *(*imports)(RzBinFile *bf);
 	RzList /*<RzBinString>*/ *(*strings)(RzBinFile *bf);
@@ -448,17 +560,18 @@ typedef struct rz_bin_plugin_t {
 	RzList /*<RzBinTrycatch>*/ *(*trycatch)(RzBinFile *bf);
 	RzList /*<RzBinClass>*/ *(*classes)(RzBinFile *bf);
 	RzList /*<RzBinMem>*/ *(*mem)(RzBinFile *bf);
-	RzList /*<RzBinReloc>*/ *(*patch_relocs)(RzBin *bin);
-	RzList /*<RzBinMap>*/ *(*maps)(RzBinFile *bf);
+	RzList /*<RzBinReloc>*/ *(*patch_relocs)(RzBinFile *bf);
 	RzList /*<RzBinFileHash>*/ *(*hashes)(RzBinFile *bf);
+	RzList /*<RzBinResource>*/ *(*resources)(RzBinFile *bf);
 	void (*header)(RzBinFile *bf);
 	char *(*signature)(RzBinFile *bf, bool json);
 	int (*demangle_type)(const char *str);
-	struct rz_bin_dbginfo_t *dbginfo;
-	struct rz_bin_write_t *write;
-	int (*get_offset)(RzBinFile *bf, int type, int idx);
-	char *(*get_name)(RzBinFile *bf, int type, int idx, bool simplified);
+	char *(*enrich_asm)(RzBinFile *bf, const char *asm_str, int asm_len);
+	ut64 (*get_offset)(RzBinFile *bf, int type, int idx);
+	char *(*get_name)(RzBinFile *bf, int type, int idx);
 	ut64 (*get_vaddr)(RzBinFile *bf, ut64 baddr, ut64 paddr, ut64 vaddr);
+	char *(*section_type_to_string)(ut64 type);
+	RzList *(*section_flag_to_rzlist)(ut64 flag);
 	RzBuffer *(*create)(RzBin *bin, const ut8 *code, int codelen, const ut8 *data, int datalen, RzBinArchOptions *opt);
 	char *(*demangle)(const char *str);
 	char *(*regstate)(RzBinFile *bf);
@@ -471,6 +584,61 @@ typedef struct rz_bin_plugin_t {
 
 typedef void (*RzBinSymbollCallback)(RzBinObject *obj, void *symbol);
 
+/**
+ * A virtual file is a binary buffer, exposed by a bin plugin for a loaded file.
+ * These virtual files can be used whenever data that is related to the file but
+ * not directly represented-as is in the raw file should be mapped into the virtual
+ * address space.
+ * Common examples for this include compressed segments or patching relocations.
+ * The idea is that the bin plugin exposes virtual files and then refers to them
+ * in the RzBinMap it returns.
+ *
+ * For example, when there is a binary format that contains a compressed segment
+ * called "text", the bin plugin would create a virtual file:
+ *
+ * 	   RzBinVirtualFile {
+ * 	     .name = "text_decompressed",
+ * 	     .buf = rz_buf_new_with_bytes(<decompressed bytes>, <decompressed size>),
+ * 	     ...
+ * 	   }
+ *
+ * which it can then use for mapping by referring to its exact name:
+ *
+ *     RzBinMap {
+ *       .vsize = <decompressed size>,
+ *       .name = "text",
+ *       .vfile_name = "text_decompressed",
+ *       ...
+ *     }
+ *
+ * When RzBin is used as part of RzCore, these virtual files can be opened as RzIO
+ * files using an URI like `vfile://<binfile id>/<filename>`. By default, RzCore
+ * sets everything up automatically though so it is rather rare that one has to
+ * manually work with these URIs.
+ */
+typedef struct rz_bin_virtual_file_t {
+	RZ_OWN RZ_NONNULL char *name;
+	RZ_NONNULL RzBuffer *buf;
+	bool buf_owned; ///< whether buf is owned and freed by this RzBinVirtualFile
+} RzBinVirtualFile;
+
+/// Description of a single memory mapping into virtual memory from a binary
+typedef struct rz_bin_map_t {
+	ut64 paddr; ///< address of the map inside the file
+	ut64 psize; ///< size of the data inside the file
+	ut64 vaddr; ///< address in the destination address space to map to
+	ut64 vsize; ///< size to map in the destination address space. If vsize > psize, excessive bytes are meant to be filled with 0
+	RZ_NULLABLE char *name;
+	ut32 perm;
+
+	/**
+	 * If not NULL, the data will be taken from the virtual file returned by the
+	 * plugin's virtual_file callback matching the given name.
+	 * If NULL, the mapping will simply be taken from the raw file.
+	 */
+	RZ_NULLABLE char *vfile_name;
+} RzBinMap;
+
 typedef struct rz_bin_section_t {
 	char *name;
 	ut64 size;
@@ -478,16 +646,27 @@ typedef struct rz_bin_section_t {
 	ut64 vaddr;
 	ut64 paddr;
 	ut32 perm;
+	ut64 align;
 	// per section platform info
 	const char *arch;
+	ut64 type;
+	ut64 flags;
 	char *format;
 	int bits;
 	bool has_strings;
-	bool add; // indicates when you want to add the section to io `S` command
 	bool is_data;
 	bool is_segment;
 } RzBinSection;
 
+/**
+ * Structure to associate a segment with the list of sections that fall in that
+ * segment.
+ */
+typedef struct rz_bin_section_map_t {
+	const RzBinSection *segment;
+	RzPVector sections;
+} RzBinSectionMap;
+
 typedef struct rz_bin_class_t {
 	char *name;
 	// TODO: char *module;
@@ -506,10 +685,12 @@ typedef struct rz_bin_class_t {
 
 #define REBASE_PADDR(o, l, type_t) \
 	do { \
-		RzListIter *_it; \
-		type_t *_el; \
-		rz_list_foreach ((l), _it, _el) { \
-			_el->paddr += (o)->loadaddr; \
+		if ((o)->opts.loadaddr) { \
+			RzListIter *_it; \
+			type_t *_el; \
+			rz_list_foreach ((l), _it, _el) { \
+				_el->paddr += (o)->opts.loadaddr; \
+			} \
 		} \
 	} while (0)
 
@@ -526,9 +707,9 @@ typedef struct rz_bin_symbol_t {
 	const char *rtype;
 	bool is_imported;
 	/* only used by java */
-	const char *visibility_str;
+	char *visibility_str;
 	// ----------------
-	//char descriptor[RZ_BIN_SIZEOF_STRINGS+1];
+	// char descriptor[RZ_BIN_SIZEOF_STRINGS+1];
 	ut64 vaddr;
 	ut64 paddr;
 	ut32 size;
@@ -552,23 +733,44 @@ typedef struct rz_bin_import_t {
 } RzBinImport;
 
 typedef struct rz_bin_reloc_t {
-	ut8 type;
-	ut8 additive;
+	RzBinRelocType type;
 	RzBinSymbol *symbol;
 	RzBinImport *import;
 	st64 addend;
-	ut64 vaddr;
-	ut64 paddr;
+	ut64 vaddr; ///< the vaddr where the value should be patched into
+	ut64 paddr; ///< the paddr where the value should be patched into
+	ut64 target_vaddr; ///< the target address that the patched reloc points to
 	ut32 visibility;
+	bool additive;
 	/* is_ifunc: indirect function, `addend` points to a resolver function
 	 * that returns the actual relocation value, e.g. chooses
 	 * an optimized version depending on the CPU.
 	 * cf. https://gcc.gnu.org/onlinedocs/gcc/Common-Function-Attributes.html
 	 */
 	bool is_ifunc;
-	RBNode vrb;
 } RzBinReloc;
 
+RZ_API ut64 rz_bin_reloc_size(RzBinReloc *reloc);
+
+/// Efficient storage of relocations to query by address
+struct rz_bin_reloc_storage_t {
+	RzBinReloc **relocs; ///< all relocs, ordered by their vaddr
+	size_t relocs_count;
+	RzBinReloc **target_relocs; ///< all relocs that have a valid target_vaddr, ordered by their target_vaddr. size is target_relocs_count!
+	size_t target_relocs_count;
+}; // RzBinRelocStorage
+
+RZ_API RzBinRelocStorage *rz_bin_reloc_storage_new(RZ_OWN RzList *relocs);
+RZ_API void rz_bin_reloc_storage_free(RzBinRelocStorage *storage);
+RZ_API RzBinReloc *rz_bin_reloc_storage_get_reloc_in(RzBinRelocStorage *storage, ut64 vaddr, ut64 size);
+
+/// return true iff there is at least one reloc in the storage with a target address
+static inline bool rz_bin_reloc_storage_targets_available(RzBinRelocStorage *storage) {
+	return storage->target_relocs_count != 0;
+}
+
+RZ_API RzBinReloc *rz_bin_reloc_storage_get_reloc_to(RzBinRelocStorage *storage, ut64 vaddr);
+
 typedef struct rz_bin_string_t {
 	// TODO: rename string->name (avoid colisions)
 	char *string;
@@ -577,7 +779,7 @@ typedef struct rz_bin_string_t {
 	ut32 ordinal;
 	ut32 size; // size of buffer containing the string in bytes
 	ut32 length; // length of string in chars
-	char type; // Ascii Wide cp850 utf8 base64 ...
+	char type; // Ascii Wide cp850 utf8 mutf8 base64 ...
 } RzBinString;
 
 typedef struct rz_bin_field_t {
@@ -588,6 +790,7 @@ typedef struct rz_bin_field_t {
 	ut32 visibility;
 	char *name;
 	char *type;
+	char *visibility_str;
 	char *comment;
 	char *format;
 	bool format_named; // whether format is the name of a format or a raw pf format string
@@ -595,41 +798,31 @@ typedef struct rz_bin_field_t {
 } RzBinField;
 
 RZ_API RzBinField *rz_bin_field_new(ut64 paddr, ut64 vaddr, int size, const char *name, const char *comment, const char *format, bool format_named);
-RZ_API void rz_bin_field_free(void *);
+RZ_API void rz_bin_field_free(RzBinField *);
 
 typedef struct rz_bin_mem_t {
 	char *name;
 	ut64 addr;
 	int size;
 	int perms;
-	RzList *mirrors; //for mirror access; stuff here should only create new maps not new fds
+	RzList /*<RzBinMem>*/ *mirrors; // for mirror access; stuff here should only create new maps not new fds
 } RzBinMem;
 
-typedef struct rz_bin_map_t {
-	ut64 addr;
-	ut64 offset;
-	int size;
-	int perms;
-	char *file;
-} RzBinMap;
-
-typedef struct rz_bin_dbginfo_t {
-	bool (*get_line)(RzBinFile *arch, ut64 addr, char *file, int len, int *line);
-} RzBinDbgInfo;
-
-typedef struct rz_bin_write_t {
-	ut64 (*scn_resize)(RzBinFile *bf, const char *name, ut64 size);
-	bool (*scn_perms)(RzBinFile *bf, const char *name, int perms);
-	int (*rpath_del)(RzBinFile *bf);
-	bool (*entry)(RzBinFile *bf, ut64 addr);
-	bool (*addlib)(RzBinFile *bf, const char *lib);
-} RzBinWrite;
+typedef struct rz_bin_resource_t {
+	size_t index;
+	char *name;
+	char *time;
+	ut64 vaddr;
+	ut64 size;
+	char *type;
+	char *language;
+} RzBinResource;
 
 // TODO: deprecate rz_bin_is_big_endian
 // TODO: has_dbg_syms... maybe flags?
 
-typedef int (*RzBinGetOffset)(RzBin *bin, int type, int idx);
-typedef const char *(*RzBinGetName)(RzBin *bin, int type, int idx, bool sd);
+typedef ut64 (*RzBinGetOffset)(RzBin *bin, int type, int idx);
+typedef char *(*RzBinGetName)(RzBin *bin, int type, int idx);
 typedef RzList *(*RzBinGetSections)(RzBin *bin);
 typedef RzBinSection *(*RzBinGetSectionAt)(RzBin *bin, ut64 addr);
 typedef char *(*RzBinDemangle)(RzBinFile *bf, const char *def, const char *str, ut64 vaddr, bool libs);
@@ -644,11 +837,23 @@ typedef struct rz_bin_bind_t {
 	ut32 visibility;
 } RzBinBind;
 
+RZ_API void rz_bin_virtual_file_free(RzBinVirtualFile *vfile);
+RZ_API void rz_bin_map_free(RzBinMap *map);
+RZ_API RzList *rz_bin_maps_of_file_sections(RZ_NONNULL RzBinFile *binfile);
+RZ_API RzList *rz_bin_sections_of_maps(RzList /*<RzBinMap>*/ *maps);
+RZ_API ut64 rz_bin_find_free_base_addr(RzList /*<RzBinMap>*/ *maps, ut64 align);
 RZ_IPI RzBinSection *rz_bin_section_new(const char *name);
 RZ_IPI void rz_bin_section_free(RzBinSection *bs);
+RZ_API RZ_OWN char *rz_bin_section_type_to_string(RzBin *bin, int type);
+RZ_API RZ_OWN RzList *rz_bin_section_flag_to_list(RzBin *bin, ut64 flag);
 RZ_API void rz_bin_info_free(RzBinInfo *rb);
-RZ_API void rz_bin_import_free(void *_imp);
-RZ_API void rz_bin_symbol_free(void *_sym);
+RZ_API void rz_bin_import_free(RzBinImport *imp);
+RZ_API void rz_bin_resource_free(RzBinResource *res);
+RZ_API void rz_bin_symbol_free(RzBinSymbol *sym);
+static inline bool rz_bin_reloc_has_target(RzBinReloc *reloc) {
+	return reloc->target_vaddr && reloc->target_vaddr != UT64_MAX;
+}
+RZ_API void rz_bin_reloc_free(RzBinReloc *reloc);
 RZ_API RzBinSymbol *rz_bin_symbol_new(const char *name, ut64 paddr, ut64 vaddr);
 RZ_API void rz_bin_string_free(void *_str);
 
@@ -656,40 +861,57 @@ RZ_API void rz_bin_string_free(void *_str);
 
 typedef struct rz_bin_options_t {
 	const char *pluginname;
-	ut64 baseaddr; // where the linker maps the binary in memory
-	ut64 loadaddr; // starting physical address to read from the target file
+	RzBinObjectLoadOptions obj_opts;
 	ut64 sz;
 	int xtr_idx; // load Nth binary
-	int rawstr;
 	int fd;
 	const char *filename;
 } RzBinOptions;
 
+typedef struct rz_event_bin_file_del_t {
+	RzBinFile *bf;
+} RzEventBinFileDel;
+
 RZ_API RzBinImport *rz_bin_import_clone(RzBinImport *o);
 RZ_API const char *rz_bin_symbol_name(RzBinSymbol *s);
 typedef void (*RzBinSymbolCallback)(RzBinObject *obj, RzBinSymbol *symbol);
 
+// common functionality for patching relocs
+RZ_API ut64 rz_bin_relocs_patch_find_targets_map_base(RzList /*<RzBinMap>*/ *maps, ut64 target_sz);
+
+typedef struct rz_bin_reloc_target_builder RzBinRelocTargetBuilder;
+RZ_API RzBinRelocTargetBuilder *rz_bin_reloc_target_builder_new(ut64 target_size, ut64 target_base);
+RZ_API void rz_bin_reloc_target_builder_free(RzBinRelocTargetBuilder *builder);
+RZ_API ut64 rz_bin_reloc_target_builder_get_target(RzBinRelocTargetBuilder *builder, ut64 sym);
+
+RZ_API void rz_bin_relocs_patch_maps(RZ_NONNULL RzList /* <RzBinMap> */ *maps,
+	RZ_NULLABLE RzBuffer *buf_patched, ut64 buf_patched_offset,
+	ut64 target_vfile_base, ut64 target_vfile_size,
+	RZ_NONNULL const char *vfile_name_patched, RZ_NONNULL const char *vfile_name_reloc_targets);
+
 // options functions
-RZ_API void rz_bin_options_init(RzBinOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, int rawstr);
+RZ_API void rz_bin_options_init(RzBinOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, bool patch_relocs);
 RZ_API void rz_bin_arch_options_init(RzBinArchOptions *opt, const char *arch, int bits);
 
 // open/close/reload functions
 RZ_API RzBin *rz_bin_new(void);
 RZ_API void rz_bin_free(RzBin *bin);
-RZ_API bool rz_bin_open(RzBin *bin, const char *file, RzBinOptions *opt);
-RZ_API bool rz_bin_open_io(RzBin *bin, RzBinOptions *opt);
-RZ_API bool rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt);
-RZ_API bool rz_bin_reload(RzBin *bin, ut32 bf_id, ut64 baseaddr);
+RZ_API RzBinFile *rz_bin_open(RzBin *bin, const char *file, RzBinOptions *opt);
+RZ_API RzBinFile *rz_bin_open_io(RzBin *bin, RzBinOptions *opt);
+RZ_API RzBinFile *rz_bin_open_buf(RzBin *bin, RzBuffer *buf, RzBinOptions *opt);
+RZ_API RzBinFile *rz_bin_reload(RzBin *bin, RzBinFile *bf, ut64 baseaddr);
 
 // plugins/bind functions
 RZ_API void rz_bin_bind(RzBin *b, RzBinBind *bnd);
-RZ_API bool rz_bin_add(RzBin *bin, RzBinPlugin *foo);
+RZ_API bool rz_bin_plugin_add(RzBin *bin, RzBinPlugin *foo);
 RZ_API bool rz_bin_xtr_add(RzBin *bin, RzBinXtrPlugin *foo);
 RZ_API bool rz_bin_ldr_add(RzBin *bin, RzBinLdrPlugin *foo);
-RZ_API void rz_bin_list(RzBin *bin, PJ *pj, int format);
 RZ_API bool rz_bin_list_plugin(RzBin *bin, const char *name, PJ *pj, int json);
 RZ_API RzBinPlugin *rz_bin_get_binplugin_by_bytes(RzBin *bin, const ut8 *bytes, ut64 sz);
 RZ_API RzBinPlugin *rz_bin_get_binplugin_by_buffer(RzBin *bin, RzBuffer *buf);
+RZ_API const RzBinPlugin *rz_bin_plugin_get(RZ_NONNULL RzBin *bin, RZ_NONNULL const char *name);
+RZ_API const RzBinXtrPlugin *rz_bin_xtrplugin_get(RZ_NONNULL RzBin *bin, RZ_NONNULL const char *name);
+RZ_API const RzBinLdrPlugin *rz_bin_ldrplugin_get(RZ_NONNULL RzBin *bin, RZ_NONNULL const char *name);
 RZ_API void rz_bin_force_plugin(RzBin *bin, const char *pname);
 
 // get/set various bin information
@@ -700,33 +922,53 @@ RZ_API RzBinInfo *rz_bin_get_info(RzBin *bin);
 RZ_API void rz_bin_set_baddr(RzBin *bin, ut64 baddr);
 RZ_API ut64 rz_bin_get_laddr(RzBin *bin);
 RZ_API ut64 rz_bin_get_size(RzBin *bin);
-RZ_API RzBinAddr *rz_bin_get_sym(RzBin *bin, int sym);
-RZ_API RzList *rz_bin_raw_strings(RzBinFile *a, int min);
-RZ_API RzList *rz_bin_dump_strings(RzBinFile *a, int min, int raw);
+RZ_API RzList *rz_bin_file_strings(RzBinFile *a, size_t min_length, bool raw_strings);
 
 // use RzBinFile instead
-RZ_API RzList *rz_bin_get_entries(RzBin *bin);
-RZ_API RzList *rz_bin_get_fields(RzBin *bin);
-RZ_API RzList *rz_bin_get_imports(RzBin *bin);
-RZ_API RzList *rz_bin_get_libs(RzBin *bin);
-RZ_API RBNode *rz_bin_patch_relocs(RzBin *bin);
-RZ_API RzList *rz_bin_patch_relocs_list(RzBin *bin);
-RZ_API RBNode *rz_bin_get_relocs(RzBin *bin);
-RZ_API RzList *rz_bin_get_relocs_list(RzBin *bin);
-RZ_API RzList *rz_bin_get_sections(RzBin *bin);
-RZ_API RzList *rz_bin_get_classes(RzBin *bin);
-RZ_API RzList *rz_bin_get_strings(RzBin *bin);
-RZ_API RzList *rz_bin_file_get_trycatch(RzBinFile *bf);
-RZ_API RzList *rz_bin_get_symbols(RzBin *bin);
-RZ_API RzList *rz_bin_reset_strings(RzBin *bin);
-RZ_API int rz_bin_is_string(RzBin *bin, ut64 va);
-RZ_API int rz_bin_is_big_endian(RzBin *bin);
-RZ_API int rz_bin_is_static(RzBin *bin);
-RZ_API ut64 rz_bin_get_vaddr(RzBin *bin, ut64 paddr, ut64 vaddr);
-RZ_API ut64 rz_bin_file_get_vaddr(RzBinFile *bf, ut64 paddr, ut64 vaddr);
-RZ_API ut64 rz_bin_a2b(RzBin *bin, ut64 addr);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_entries(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_fields(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_imports(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_libs(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_sections(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_classes(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_strings(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_mem(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_get_symbols(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE RZ_BORROW RzList *rz_bin_reset_strings(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE int rz_bin_is_string(RZ_NONNULL RzBin *bin, ut64 va);
+RZ_API RZ_DEPRECATE int rz_bin_is_big_endian(RZ_NONNULL RzBin *bin);
+RZ_API RZ_DEPRECATE int rz_bin_is_static(RZ_NONNULL RzBin *bin);
+RZ_API RzList *rz_bin_file_get_trycatch(RZ_NONNULL RzBinFile *bf);
+
+RZ_API const RzList *rz_bin_object_get_entries(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_fields(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_imports(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzBinInfo *rz_bin_object_get_info(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_libs(RZ_NONNULL RzBinObject *obj);
+RZ_API const RBNode *rz_bin_object_get_relocs(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_sections_all(RZ_NONNULL RzBinObject *obj);
+RZ_API RZ_OWN RzList *rz_bin_object_get_sections(RZ_NONNULL RzBinObject *obj);
+RZ_API RZ_OWN RzList *rz_bin_object_get_segments(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_classes(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_strings(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_mem(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_resources(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_get_symbols(RZ_NONNULL RzBinObject *obj);
+RZ_API const RzList *rz_bin_object_reset_strings(RZ_NONNULL RzBin *bin, RZ_NONNULL RzBinFile *bf, RZ_NONNULL RzBinObject *obj);
+RZ_API bool rz_bin_object_is_string(RZ_NONNULL RzBinObject *obj, ut64 va);
+RZ_API bool rz_bin_object_is_big_endian(RZ_NONNULL RzBinObject *obj);
+RZ_API bool rz_bin_object_is_static(RZ_NONNULL RzBinObject *obj);
+RZ_API RZ_OWN RzVector *rz_bin_object_sections_mapping_list(RZ_NONNULL RzBinObject *obj);
+RZ_API RZ_OWN RzPVector *rz_bin_object_get_maps_at(RzBinObject *o, ut64 off, bool va);
+RZ_API RZ_BORROW RzBinMap *rz_bin_object_get_map_at(RZ_NONNULL RzBinObject *o, ut64 off, bool va);
+RZ_API ut64 rz_bin_object_p2v(RZ_NONNULL RzBinObject *obj, ut64 paddr);
+RZ_API RzVector *rz_bin_object_p2v_all(RZ_NONNULL RzBinObject *obj, ut64 paddr);
+RZ_API ut64 rz_bin_object_v2p(RZ_NONNULL RzBinObject *obj, ut64 vaddr);
+
+RZ_API RzBinLanguage rz_bin_language_detect(RzBinFile *binfile);
+RZ_API RzBinLanguage rz_bin_language_to_id(const char *language);
+RZ_API const char *rz_bin_language_to_string(RzBinLanguage language);
 
-RZ_API int rz_bin_load_languages(RzBinFile *binfile);
 RZ_API RzBinFile *rz_bin_cur(RzBin *bin);
 RZ_API RzBinObject *rz_bin_cur_object(RzBin *bin);
 
@@ -734,29 +976,20 @@ RZ_API RzBinObject *rz_bin_cur_object(RzBin *bin);
 RZ_API bool rz_bin_select(RzBin *bin, const char *arch, int bits, const char *name);
 RZ_API bool rz_bin_select_bfid(RzBin *bin, ut32 bf_id);
 RZ_API bool rz_bin_use_arch(RzBin *bin, const char *arch, int bits, const char *name);
-RZ_API void rz_bin_list_archs(RzBin *bin, PJ *pj, int mode);
 RZ_API RzBuffer *rz_bin_create(RzBin *bin, const char *plugin_name, const ut8 *code, int codelen, const ut8 *data, int datalen, RzBinArchOptions *opt);
-RZ_API RzBuffer *rz_bin_package(RzBin *bin, const char *type, const char *file, RzList *files);
 
-RZ_API const char *rz_bin_string_type(int type);
+RZ_API RZ_BORROW const char *rz_bin_string_type(int type);
 RZ_API const char *rz_bin_entry_type_string(int etype);
 
-RZ_API bool rz_bin_file_object_new_from_xtr_data(RzBin *bin, RzBinFile *bf, ut64 baseaddr, ut64 loadaddr, RzBinXtrData *data);
+RZ_API bool rz_bin_file_object_new_from_xtr_data(RzBin *bin, RzBinFile *bf, RzBinObjectLoadOptions *opts, RzBinXtrData *data);
 
-// RzBinFile lifecycle
-// RZ_IPI RzBinFile *rz_bin_file_new(RzBin *bin, const char *file, ut64 file_sz, int rawstr, int fd, const char *xtrname, Sdb *sdb, bool steal_ptr);
-RZ_API bool rz_bin_file_close(RzBin *bin, int bd);
-RZ_API void rz_bin_file_free(void /*RzBinFile*/ *bf_);
 // RzBinFile.get
 RZ_API RzBinFile *rz_bin_file_at(RzBin *bin, ut64 addr);
 RZ_API RzBinFile *rz_bin_file_find_by_object_id(RzBin *bin, ut32 binobj_id);
 RZ_API RzList *rz_bin_file_get_symbols(RzBinFile *bf);
-//
-RZ_API ut64 rz_bin_file_get_vaddr(RzBinFile *bf, ut64 paddr, ut64 vaddr);
 // RzBinFile.add
 RZ_API RzBinClass *rz_bin_file_add_class(RzBinFile *binfile, const char *name, const char *super, int view);
 RZ_API RzBinSymbol *rz_bin_file_add_method(RzBinFile *bf, const char *classname, const char *name, int nargs);
-RZ_API RzBinField *rz_bin_file_add_field(RzBinFile *binfile, const char *classname, const char *name);
 // RzBinFile.find
 RZ_API RzBinFile *rz_bin_file_find_by_arch_bits(RzBin *bin, const char *arch, int bits);
 RZ_API RzBinFile *rz_bin_file_find_by_id(RzBin *bin, ut32 bin_id);
@@ -765,59 +998,44 @@ RZ_API RzBinFile *rz_bin_file_find_by_name(RzBin *bin, const char *name);
 
 RZ_API bool rz_bin_file_set_cur_binfile(RzBin *bin, RzBinFile *bf);
 RZ_API bool rz_bin_file_set_cur_by_name(RzBin *bin, const char *name);
-RZ_API bool rz_bin_file_deref(RzBin *bin, RzBinFile *a);
 RZ_API bool rz_bin_file_set_cur_by_fd(RzBin *bin, ut32 bin_fd);
 RZ_API bool rz_bin_file_set_cur_by_id(RzBin *bin, ut32 bin_id);
 RZ_API bool rz_bin_file_set_cur_by_name(RzBin *bin, const char *name);
 RZ_API ut64 rz_bin_file_delete_all(RzBin *bin);
-RZ_API bool rz_bin_file_delete(RzBin *bin, ut32 bin_id);
-RZ_API RzList *rz_bin_file_compute_hashes(RzBin *bin, ut64 limit);
+RZ_API bool rz_bin_file_delete(RzBin *bin, RzBinFile *bf);
+RZ_API RZ_OWN RzList *rz_bin_file_compute_hashes(RzBin *bin, RzBinFile *bf, ut64 limit);
 RZ_API RzList *rz_bin_file_set_hashes(RzBin *bin, RzList *new_hashes);
 RZ_API RzBinPlugin *rz_bin_file_cur_plugin(RzBinFile *binfile);
 RZ_API void rz_bin_file_hash_free(RzBinFileHash *fhash);
 
 // binobject functions
 RZ_API int rz_bin_object_set_items(RzBinFile *binfile, RzBinObject *o);
-RZ_API bool rz_bin_object_delete(RzBin *bin, ut32 binfile_id);
+RZ_API ut64 rz_bin_object_addr_with_base(RzBinObject *o, ut64 addr);
+RZ_API ut64 rz_bin_object_get_vaddr(RzBinObject *o, ut64 paddr, ut64 vaddr);
+RZ_API const RzBinAddr *rz_bin_object_get_special_symbol(RzBinObject *o, RzBinSpecialSymbol sym);
+RZ_API RzBinRelocStorage *rz_bin_object_patch_relocs(RzBinFile *bf, RzBinObject *o);
+RZ_API RzBinSymbol *rz_bin_object_get_symbol_of_import(RzBinObject *o, RzBinImport *imp);
+RZ_API RzBinVirtualFile *rz_bin_object_get_virtual_file(RzBinObject *o, const char *name);
 RZ_API void rz_bin_mem_free(void *data);
 
 // demangle functions
-RZ_API char *rz_bin_demangle(RzBinFile *binfile, const char *lang, const char *str, ut64 vaddr, bool libs);
-RZ_API char *rz_bin_demangle_java(const char *str);
-RZ_API char *rz_bin_demangle_cxx(RzBinFile *binfile, const char *str, ut64 vaddr);
-RZ_API char *rz_bin_demangle_msvc(const char *str);
-RZ_API char *rz_bin_demangle_swift(const char *s, bool syscmd);
-RZ_API char *rz_bin_demangle_objc(RzBinFile *binfile, const char *sym);
-RZ_API char *rz_bin_demangle_rust(RzBinFile *binfile, const char *str, ut64 vaddr);
-RZ_API int rz_bin_demangle_type(const char *str);
-RZ_API void rz_bin_demangle_list(RzBin *bin);
-RZ_API char *rz_bin_demangle_plugin(RzBin *bin, const char *name, const char *str);
+RZ_API RZ_OWN char *rz_bin_demangle(RZ_NULLABLE RzBinFile *bf, RZ_NULLABLE const char *language, RZ_NULLABLE const char *symbol, ut64 vaddr, bool libs);
 RZ_API const char *rz_bin_get_meth_flag_string(ut64 flag, bool compact);
 
-RZ_API RzBinSection *rz_bin_get_section_at(RzBinObject *o, ut64 off, int va);
+RZ_API RZ_BORROW RzBinSection *rz_bin_get_section_at(RzBinObject *o, ut64 off, int va);
 
 /* dbginfo.c */
-RZ_API bool rz_bin_addr2line(RzBin *bin, ut64 addr, char *file, int len, int *line);
-RZ_API char *rz_bin_addr2text(RzBin *bin, ut64 addr, int origin);
-RZ_API char *rz_bin_addr2fileline(RzBin *bin, ut64 addr);
-/* bin_write.c */
-RZ_API bool rz_bin_wr_addlib(RzBin *bin, const char *lib);
-RZ_API ut64 rz_bin_wr_scn_resize(RzBin *bin, const char *name, ut64 size);
-RZ_API bool rz_bin_wr_scn_perms(RzBin *bin, const char *name, int perms);
-RZ_API bool rz_bin_wr_rpath_del(RzBin *bin);
-RZ_API bool rz_bin_wr_entry(RzBin *bin, ut64 addr);
-RZ_API bool rz_bin_wr_output(RzBin *bin, const char *filename);
-
-RZ_API RzList *rz_bin_get_mem(RzBin *bin);
+RZ_DEPRECATE RZ_API bool rz_bin_addr2line(RzBin *bin, ut64 addr, char *file, int len, int *line);
+RZ_DEPRECATE RZ_API char *rz_bin_addr2text(RzBin *bin, ut64 addr, int origin);
 
 /* filter.c */
 RZ_API void rz_bin_load_filter(RzBin *bin, ut64 rules);
 RZ_API void rz_bin_filter_symbols(RzBinFile *bf, RzList *list);
 RZ_API void rz_bin_filter_sections(RzBinFile *bf, RzList *list);
-RZ_API char *rz_bin_filter_name(RzBinFile *bf, Sdb *db, ut64 addr, char *name);
+RZ_API char *rz_bin_filter_name(RzBinFile *bf, HtPU *db, ut64 addr, char *name);
 RZ_API void rz_bin_filter_sym(RzBinFile *bf, HtPP *ht, ut64 vaddr, RzBinSymbol *sym);
 RZ_API bool rz_bin_strpurge(RzBin *bin, const char *str, ut64 addr);
-RZ_API bool rz_bin_string_filter(RzBin *bin, const char *str, ut64 addr);
+RZ_API bool rz_bin_string_filter(RzBin *bin, const char *str, int len, ut64 addr);
 
 /* plugin pointers */
 extern RzBinPlugin rz_bin_plugin_any;
@@ -828,6 +1046,7 @@ extern RzBinPlugin rz_bin_plugin_elf64;
 extern RzBinPlugin rz_bin_plugin_p9;
 extern RzBinPlugin rz_bin_plugin_ne;
 extern RzBinPlugin rz_bin_plugin_le;
+extern RzBinPlugin rz_bin_plugin_luac;
 extern RzBinPlugin rz_bin_plugin_pe;
 extern RzBinPlugin rz_bin_plugin_mz;
 extern RzBinPlugin rz_bin_plugin_pe64;
